part of jaguar_oauth2;

/// Wraps the route with [LinkFacebook] interceptor that links facebook
/// with existing user account
class WrapLinkFacebook extends RouteWrapper<LinkFacebook> {
  final FacebookModelManager manager;

  final String id;

  const WrapLinkFacebook(this.manager, {this.id});

  LinkFacebook createInterceptor() => new LinkFacebook(this.manager);
}

/// Links facebook with existing user account
class LinkFacebook extends Interceptor {
  final FacebookModelManager manager;

  LinkFacebook(this.manager);

  Future<FacebookUserModel> pre(@Input(OAuth2Authorized) oauth2.Client client,
      @Input(UserAuthoriser) UserModel model) async {
    if (model is! FacebookUserModel) {
      throw new UnAuthorizedError();
    }

    final graph = new fb.GraphApi(client);
    final fields = new fb.UserFieldSelector()
      ..addBirthday()
      ..addAbout()
      ..addEmail()
      ..addName();
    final fb.UserResult resp = await graph.getMe(fields: fields);
    await manager.setFbInfo(model, resp.id, client.credentials.accessToken,
        client.credentials.refreshToken);
    final ret =
        await manager.fetchModelByAuthorizationId(model.authenticationId);
    if (ret == null) {
      throw new UnAuthorizedError();
    }
    return ret;
  }
}
